---------------------------------------------------------------------------
-- С ɽ⥸塼
-- 
-- Copyright 2009 (c) Takefumi MIYOSHI, All Rights Reserved. 
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.global.all;

entity input_rs232c is
  generic (
    sys_clk : integer := 14000000;
    rate : integer := 9600;
    BUFSIZE : integer := 32
    );
  port (
    pClk : in std_logic;   -- ƥ९å
    pReq  : in std_logic;  -- ɽꥯ
    pBusy : out std_logic; -- ӥ
    pTxD : out std_logic;  -- ǡ
    pRxD : in std_logic;  -- ǡ
    pTurn: in GameTurn;  -- 
    pX : out std_logic_vector(2 downto 0);    -- ϤȤƼȤäXɸ
    pY : out std_logic_vector(2 downto 0);    -- ϤȤƼȤäYɸ
    pReset : in std_logic  -- ꥻå
    );
end input_rs232c;

architecture rtl of input_rs232c is

  component rs232c_tx
    generic (
      sys_clk : integer := 14000000;
      rate : integer := 9600
      );
    port (
      clk   : in  std_logic;
      rst   : in  std_logic;
      wr    : in  std_logic;
      din   : in  std_logic_vector(7 downto 0);
      dout  : out std_logic;
      ready : out std_logic
      );
  end component;

  component rs232c_rx
    generic(
      sys_clk : integer := 14000000;
      rate : integer := 9600
      );
    port(
      clk  : in  std_logic;
      rst  : in  std_logic;
      din  : in  std_logic;
      rd   : out std_logic;
      dout : out std_logic_vector(7 downto 0)
      );
  end component;

  -- rs232c_txȤ³뤿ο
  signal cWr    : std_logic;
  signal cData  : std_logic_vector(7 downto 0);
  signal cReady : std_logic;

  -- rs232c_rxȤ³뤿ο
  signal cRd   : std_logic;
  signal cDout : std_logic_vector(7 downto 0);
  signal dRd   : std_logic;
  
  -- ϥХåե
  type reversi_string is array (0 to BUFSIZE-1) of reversi_char;
  signal cInputBuffer : reversi_string;

  -- ѿ
  type StateType is (IDLE, PROMPT, INPUT_READ, PARSE, PRINT_CRLF);
  signal cState : StateType;
  signal cNextState : StateType;

  signal cCounter : std_logic_vector(4 downto 0);

  signal cX : std_logic_vector(2 downto 0);
  signal cY : std_logic_vector(2 downto 0);
  signal cLastFlag : std_logic;
  
  -- (CRLF)
  constant cCR  : reversi_char := X"0d";
  constant cLF  : reversi_char := X"0a";
  
begin

  pX <= cX;
  pY <= cY;
  
  U0: rs232c_tx
    generic map(
      sys_clk => sys_clk,
      rate    => rate
      )
    port map (
      clk   => pClk,
      rst   => pReset,
      wr    => cWr,
      din   => cData,
      dout  => pTxD,
      ready => cReady
    );

  U1: rs232c_rx
    generic map(
      sys_clk => sys_clk,
      rate => rate
      )
    port map(
      clk  => pClk,
      rst  => pReset,
      din  => pRxD,
      rd   => cRd,
      dout => cDout
      );

  process (pClk, pReset)
  begin
    if pReset = '1' then
      pBusy   <= '0';
      cWr     <= '0';
      cData   <= (others => '0');
      cState  <= IDLE;
      cCounter <= (others => '0');
    elsif pClk'event and pClk = '1' then
      case cState is
        when IDLE =>
          cWr     <= '0';
          cData   <= (others => '0');
          cCounter <= (others => '0');
          cLastFlag <= '0';
          if pReq = '1' then
            cState <= PROMPT;
            pBusy <= '1';
          else
            pBusy <= '0';
          end if;
        when PRINT_CRLF =>
          if(cReady = '1' and cWr = '0') then
            if(conv_integer(cCounter) = 1) then
              if cNextState = PRINT_CRLF then
                cNextState <= IDLE;
              end if;
              cState <= cNextState;
              cCounter <= (others => '0');
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            if(conv_integer(cCounter) = 0) then
              cData <= cCR;
            elsif(conv_integer(cCounter) = 1) then
              cData <= cLF;
            end if;
          else
            cWr <= '0';
          end if;
        when PROMPT =>
          if(cReady = '1' and cWr = '0') then
            if(conv_integer(cCounter) = 1) then
              cNextState <= INPUT_READ;
              cState <= PRINT_CRLF;
              cCounter <= (others => '0');
              cInputBuffer <= (others => X"00");
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            if(conv_integer(cCounter) = 0) then
              if pTurn = WHITE then
                cData <= X"57"; -- (format "%x" ?W)
              elsif pTurn = BLACK then
                cData <= X"42"; -- (format "%x" ?B)
              end if;
            elsif(conv_integer(cCounter) = 1) then
              cData <= X"3f"; -- (format "%x" ??)
            end if;
          else
            cWr <= '0';
          end if;
        when INPUT_READ =>
          dRd <= cRd;
          if cRd = '0' and dRd = '1' then
            if cDout = X"0a" or cDout = X"0d" or (conv_integer(cCounter) = BUFSIZE) then
              cNextState <= PARSE;
              cState <= PRINT_CRLF;
              cCounter <= (others => '0');
            else
              cInputBuffer(conv_integer(cCounter)) <= cDout;
              cCounter <= cCounter + 1;
              cWr <= '1';
              cData <= cDout;
            end if;
          else
            cWr <= '0';
          end if;
        when PARSE =>
          cWr <= '0';
          if(cInputBuffer(0) < X"61" or cInputBuffer(0) > X"68" or cInputBuffer(1) < X"31" or cInputBuffer(1) > X"38") then
            cNextState <= PROMPT;
            cState <= PRINT_CRLF;
            cCounter <= (others => '0');
            cInputBuffer <= (others => X"00");
          else
            case cInputBuffer(0) is
              when X"61" => cX <= "000";
              when X"62" => cX <= "001";
              when X"63" => cX <= "010";
              when X"64" => cX <= "011";
              when X"65" => cX <= "100";
              when X"66" => cX <= "101";
              when X"67" => cX <= "110";
              when X"68" => cX <= "111";
              when others => null;
            end case;
            case cInputBuffer(1) is
              when X"31" => cY <= "000";
              when X"32" => cY <= "001";
              when X"33" => cY <= "010";
              when X"34" => cY <= "011";
              when X"35" => cY <= "100";
              when X"36" => cY <= "101";
              when X"37" => cY <= "110";
              when X"38" => cY <= "111";
              when others => null;
            end case;
--            cX <= cInputBuffer(0)(2 downto 0) - 1;
--            cY <= cInputBuffer(1)(2 downto 0) - 1;
            cNextState <= PRINT_CRLF;
            cState <= PRINT_CRLF;
            cCounter <= (others => '0');
          end if;
        when others =>
          cState <= IDLE;
      end case;
    end if;
  end process;

end rtl;

