---------------------------------------------------------------------------
-- o[V \W[
-- 
-- Copyright 2009 (c) Takefumi MIYOSHI, All Rights Reserved. 
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.global.all;

entity print_rs232c is
  generic (
    sys_clk : integer := 14000000;
    rate : integer := 9600;
    LINE_STR_LEN : integer := (8*2+2)
    );
  port (
    pClk : in std_logic;   -- VXeNbN
    pWhiteData : in std_logic_vector(63 downto 0); -- ΂̃rbge[u
    pBlackData : in std_logic_vector(63 downto 0); -- ΂̃rbge[u
    pInputX : in std_logic_vector(2 downto 0);
    pInputY : in std_logic_vector(2 downto 0);
    pTurn   : in GameTurn;
    pReq  : in std_logic;  -- \NGXg
    pBusy : out std_logic; -- rW[M
    pTxD : out std_logic;  -- f[^o
    pReset : in std_logic  -- Zbg
    );
end print_rs232c;

architecture rtl of print_rs232c is

  component rs232c_tx
    generic (
      sys_clk : integer := 14000000;
      rate : integer := 9600
      );
    port (
      clk   : in  std_logic;
      rst   : in  std_logic;
      wr    : in  std_logic;
      din   : in  std_logic_vector(7 downto 0);
      dout  : out std_logic;
      ready : out std_logic
      );
  end component;

  -- rs232c_txƂ̐ڑ邽߂̐M
  signal cWr    : std_logic;
  signal cData  : std_logic_vector(7 downto 0);
  signal cReady : std_logic;
  
  -- ԕϐ
  type StateType is (IDLE, PRINT_INPUT, PRINT_LABEL, PRINT_BAR, PRINT_VAL, PRINT_CRLF);
  signal cState : StateType;
  signal cNextState : StateType;

  signal cCounter : std_logic_vector(4 downto 0);

  signal cX : std_logic_vector(2 downto 0);
  signal cY : std_logic_vector(2 downto 0);
  signal cLastFlag : std_logic;
  
  type reversi_string is array (0 to LINE_STR_LEN-1) of reversi_char;

  -- ̍Wx(a,b,c,d,e,f,g,h)
  constant cHorizontalLabel : reversi_string :=
    (X"20", X"20", X"61", X"20", X"62", X"20", X"63", X"20", X"64",
     X"20", X"65", X"20", X"66", X"20", X"67", X"20", X"68", X"20");
  -- c̍W(1,2,3,4,5,6,7,8)
  constant cVerticalLabel : reversi_string :=
    (X"31", X"32", X"33", X"34", X"35", X"36", X"37", X"38", X"20",
     X"20", X"20", X"20", X"20", X"20", X"20", X"20", X"20", X"20");
  -- ̐
  constant cHorizontalBar : reversi_string :=
    (X"20", X"2b", X"2d", X"2b", X"2d", X"2b", X"2d", X"2b", X"2d",
     X"2b", X"2d", X"2b", X"2d", X"2b", X"2d", X"2b", X"2d", X"2b");
  -- ̘g(-)
  constant cHBar  : reversi_char := X"2d";
  -- c̘g(|)
  constant cVBar  : reversi_char := X"7c";
  -- _(+)
  constant cCross : reversi_char := X"2b";
  -- ]( )
  constant cSpace : reversi_char := X"20";
  -- (*)
  constant cBlackMark : reversi_char := X"2a";
  -- (O)
  constant cWhiteMark : reversi_char := X"4f";
  -- s(CRLF)
  constant cCR  : reversi_char := X"0d";
  constant cLF  : reversi_char := X"0a";

  -- JID𕶎ɕϊ֐
  function COLUMN_ID (v : in std_logic_vector)
    return std_logic_vector is
    variable DATA : std_logic_vector(7 downto 0);
  begin
    case conv_integer(v) is
      when 0 => DATA := X"61";
      when 1 => DATA := X"62";
      when 2 => DATA := X"63";
      when 3 => DATA := X"64";
      when 4 => DATA := X"65";
      when 5 => DATA := X"66";
      when 6 => DATA := X"67";
      when 7 => DATA := X"68";
      when others => DATA := X"00";
    end case;
    return DATA;
  end COLUMN_ID;

  -- sID𕶎ɕϊ֐
  function ROW_ID (v : in std_logic_vector)
    return std_logic_vector is
    variable DATA : std_logic_vector(7 downto 0);
  begin
    case conv_integer(v) is
      when 0 => DATA := X"31";
      when 1 => DATA := X"32";
      when 2 => DATA := X"33";
      when 3 => DATA := X"34";
      when 4 => DATA := X"35";
      when 5 => DATA := X"36";
      when 6 => DATA := X"37";
      when 7 => DATA := X"38";
      when others => DATA := X"00";
    end case;
    return DATA;
  end ROW_ID;
  
begin

  U0: rs232c_tx
    generic map(
      sys_clk => sys_clk,
      rate    => rate
      )
    port map (
      clk   => pClk,
      rst   => pReset,
      wr    => cWr,
      din   => cData,
      dout  => pTxD,
      ready => cReady
    );

  process (pClk, pReset)
  begin
    if pReset = '1' then
      pBusy   <= '0';
      cWr     <= '0';
      cData   <= (others => '0');
      cState  <= IDLE;
      cCounter <= (others => '0');
      cX <= (others => '0');
      cY <= (others => '0');
      cLastFlag <= '0';
    elsif pClk'event and pClk = '1' then
      case cState is
        when IDLE =>
          cWr     <= '0';
          cData   <= (others => '0');
          cCounter <= (others => '0');
          cX <= (others => '0');
          cY <= (others => '0');
          cLastFlag <= '0';
          if pReq = '1' then
            cState <= PRINT_INPUT;
            pBusy   <= '1';
          else
            pBusy   <= '0';
          end if;
        when PRINT_CRLF =>
          if(cReady = '1' and cWr = '0') then
            if(conv_integer(cCounter) = 1) then
              if cNextState = PRINT_CRLF then
                cNextState <= IDLE;
              end if;
              cState <= cNextState;
              cCounter <= (others => '0');
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            if(conv_integer(cCounter) = 0) then
              cData <= cCR;
            elsif(conv_integer(cCounter) = 1) then
              cData <= cLF;
            end if;
          else
            cWr <= '0';
          end if;
        when PRINT_INPUT =>
          if(cReady = '1' and cWr = '0') then
            if (conv_integer(cCounter) = 3) then
              cCounter <= (others => '0');
              cNextState <= PRINT_LABEL;
              cState <= PRINT_CRLF;
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            case conv_integer(cCounter) is
              when 0 =>
                if pTurn = WHITE then
                  cData <= X"42";       -- OB(0x42)̃^[
                else
                  cData <= X"57";       -- OW(0x57)̃^[
                end if;
              when 1 => cData <= X"3a";  -- :
              when 2 => cData <= COLUMN_ID(pInputX);
              when 3 => cData <= ROW_ID(pInputY);
              when others => null;
            end case;
          else
            cWr <= '0';
          end if;
        when PRINT_LABEL =>
          if(cReady = '1' and cWr = '0') then
            if (conv_integer(cCounter) = LINE_STR_LEN-1) then
              cCounter <= (others => '0');
              cNextState <= PRINT_BAR;
              cState <= PRINT_CRLF;
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            cData <= cHorizontalLabel(conv_integer(cCounter));
          else
            cWr <= '0';
          end if;
        when PRINT_BAR   =>
          if(cReady = '1' and cWr = '0') then
            if (conv_integer(cCounter) = LINE_STR_LEN-1) then
              cCounter <= (others => '0');
              if(cLastFlag = '1') then
                cNextState <= PRINT_CRLF;
              else
                cNextState <= PRINT_VAL;
              end if;
              cX <= (others => '0');
              cState <= PRINT_CRLF;
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            cData <= cHorizontalBar(conv_integer(cCounter));
          else
            cWr <= '0';
          end if;
        when PRINT_VAL   =>
          if(cReady = '1' and cWr = '0') then
            if (conv_integer(cCounter) = LINE_STR_LEN-1) then
              cCounter <= (others => '0');
              cNextState <= PRINT_BAR;
              cState <= PRINT_CRLF;
              cY <= cY + 1;
              if(cY = "111") then
                cLastFlag <= '1';
              end if;
            else
              cCounter <= cCounter + 1;
            end if;
            cWr <= '1';
            if (conv_integer(cCounter) = 0) then
              cData <= cVerticalLabel(conv_integer(cY));
            elsif (cCounter(0) = '1') then
              cData <= cVBar;
            else
              cX <= cX + 1;
              if (pBlackData(conv_integer(cX&cY)) = '1') then
                cData <= cBlackMark;
              elsif (pWhiteData(conv_integer(cX&cY)) = '1') then
                cData <= cWhiteMark;
              else
                cData <= cSpace;
              end if;
            end if;
          else
            cWr <= '0';
          end if;
        when others =>
          cState <= IDLE;
      end case;
    end if;
  end process;
  
end rtl;

