--
--  o[V gbvW[
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.global.all;

entity reversi is
  generic (
    sys_clk : integer := 50000000;
    rate : integer    := 19200;
    MAX_TRY_NUM       : integer := 2000
    );
  port (
    pClk   : in  std_logic;
    pRxD   : in  std_logic;
    pTxD   : out std_logic;
    pReset : in  std_logic
    );
end reversi;

architecture rtl of reversi is

  component print_rs232c
    generic (
      sys_clk : integer := 14000000;
      rate : integer := 9600;
      LINE_STR_LEN : integer := (8*2+2)
      );
    port (
      pClk : in std_logic;   -- VXeNbN
      pWhiteData : in std_logic_vector(63 downto 0); -- ΂̃rbge[u
      pBlackData : in std_logic_vector(63 downto 0); -- ΂̃rbge[u
      pInputX : in std_logic_vector(2 downto 0);
      pInputY : in std_logic_vector(2 downto 0);
      pTurn   : in GameTurn;
      pReq  : in std_logic;  -- \NGXg
      pBusy : out std_logic; -- rW[M
      pTxD : out std_logic;  -- f[^o
      pReset : in std_logic  -- Zbg
      );
  end component;

  component input_rs232c
  generic (
    sys_clk : integer := 14000000;
    rate : integer := 9600;
    BUFSIZE : integer := 32
    );
  port (
    pClk : in std_logic;   -- VXeNbN
    pReq  : in std_logic;  -- \NGXg
    pBusy : out std_logic; -- rW[M
    pTxD : out std_logic;  -- f[^o
    pRxD : in std_logic;  -- f[^o
    pTurn : in GameTurn;
    pX : out std_logic_vector(2 downto 0);    -- ͂ƂĎ󂯂ƂXW
    pY : out std_logic_vector(2 downto 0);    -- ͂ƂĎ󂯂ƂYW
    pReset : in std_logic  -- Zbg
    );
  end component;

  component reversi_judge
    port (
      Ax : in std_logic_vector(2 downto 0);
      Ay : in std_logic_vector(2 downto 0);
      Turn : in GameTurn;
      BlackData : in std_logic_vector(63 downto 0);
      WhiteData : in std_logic_vector(63 downto 0);
      ResultData : out std_logic_vector(63 downto 0)
      );
  end component;

  component xorshift32
    port (
      pClk   : in  std_logic;
      pReset : in  std_logic;
      Q      : out std_logic_vector(31 downto 0)
      );
  end component;

  signal cWhiteData : std_logic_vector(63 downto 0); -- ΂̃rbge[u
  signal cBlackData : std_logic_vector(63 downto 0); -- ΂̃rbge[u

  signal cPrintReq  : std_logic;  -- \NGXg
  signal cPrintBusy : std_logic; -- rW[M
  signal cPrintTxD  : std_logic;

  signal cInputReq  : std_logic;
  signal cInputBusy : std_logic;
  signal cInputTxD  : std_logic;
  signal cUserInputX    : std_logic_vector(2 downto 0);
  signal cUserInputY    : std_logic_vector(2 downto 0);

  signal cInputX    : std_logic_vector(2 downto 0);
  signal cInputY    : std_logic_vector(2 downto 0);

  signal cReset : std_logic;

  type GameState is (IDLE, PRINT, INPUT_WAIT, UPDATE, COMP);
  signal cState : GameState;
  signal cStateCounter : std_logic_vector(1 downto 0);

  signal cGameTurn : GameTurn;
  signal cOppositeGameTurn : GameTurn;

  signal cReversiResultData : std_logic_vector(63 downto 0);
  signal cReversiResultFlag : std_logic;

  signal cXORSHIFT : std_logic_vector(5 downto 0);
  signal cSelectCounter : std_logic_vector(10 downto 0);

  constant L : std_logic := '0';

begin  -- rtl

  cReset <= pReset;

  cOppositeGameTurn <= BLACK when cGameTurn = WHITE else WHITE;

  U_PRINT: print_rs232c
    generic map(
      sys_clk => sys_clk,
      rate => rate
      )
    port map(
      pClk => pClk,
      pWhiteData => cWhiteData,
      pBlackData => cBlackData,
      pInputX => cInputX,
      pInputY => cInputY,
      pTurn   => cGameTurn,
      pReq       => cPrintReq,
      pBusy      => cPrintBusy,
      pTxD       => cPrintTxD,
      pReset     => cReset
      );

  U_INPUT: input_rs232c
    generic map(
      sys_clk => sys_clk,
      rate => rate
      )
    port map(
      pClk   => pClk,
      pReq   => cInputReq,
      pBusy  => cInputBusy,
      pTxD   => cInputTxD,
      pRxD   => pRxD,
      pTurn  => cGameTurn,
      pX     => cUserInputX,
      pY     => cUserInputY,
      pReset => cReset
      );

  U_JUDGE: reversi_judge
    port map(
      Ax => cInputX,
      Ay => cInputY,
      Turn => cGameTurn,
      BlackData => cBlackData,
      WhiteData => cWhiteData,
      ResultData => cReversiResultData
      );

  U_XORSHIFT : xorshift32
    port map(
      pClk    => pClk,
      pReset  => cReset,
      Q(5 downto 0) => cXORSHIFT,
      Q(31 downto 6) => open
      );

  process(cInputTxD, cPrintTxD, cState)
  begin
    case cState is
      when PRINT      => pTxD <= cPrintTxD;
      when INPUT_WAIT => pTxD <= cInputTxD;
      when others     => pTxD <= '1';
    end case;
  end process;
  
  process (pClk, cReset)
  begin
    if (cReset = '1') then
      cPrintReq <= '0';
      cState <= IDLE;
      cStateCounter <= (others => '0');
      cGameTurn <= BLACK;
      cSelectCounter <= (others => '0');
      cInputX <= (others => '0');
      cInputY <= (others => '0');
      for i in 0 to 63 loop
        case i is
          when 28 | 35 =>
            cBlackData(i) <= '1';
            cWhiteData(i) <= '0';
          when 27 | 36 =>
            cBlackData(i) <= '0';
            cWhiteData(i) <= '1';
          when others =>
            cWhiteData(i) <= '0';
            cBlackData(i) <= '0';
        end case;
      end loop;  -- i
    elsif (pClk'event and pClk = '1') then
      case cState is
        when IDLE =>
          cPrintReq <= '0';
          cState <= PRINT;
          cStateCounter <= (others => '0');
          cGameTurn <= BLACK;
          cSelectCounter <= (others => '0');
          for i in 0 to 63 loop
            case i is
              when 28 | 35 =>
                cBlackData(i) <= '1';
                cWhiteData(i) <= '0';
              when 27 | 36 =>
                cBlackData(i) <= '0';
                cWhiteData(i) <= '1';
              when others =>
                cWhiteData(i) <= '0';
                cBlackData(i) <= '0';
            end case;
          end loop;  -- i
        when PRINT =>
          case conv_integer(cStateCounter) is
            when 0 =>
              cPrintReq <= '1';
              cStateCounter <= cStateCounter + 1;
            when 1 =>
              if cPrintBusy = '1' then
                cStateCounter <= cStateCounter + 1;
              else
                cPrintReq <= '0';
              end if;  
            when 2 =>
              cPrintReq <= '0';
              if cPrintBusy = '0' then
                cStateCounter <= (others => '0');
                if cGameTurn = BLACK then
                  -- [U̓͑҂
                  cState <= INPUT_WAIT;
                else
                  -- Rs[^̓
                  cState <= COMP;
                end if;
              end if;
            when others =>
              cState <= IDLE;
          end case;
        when INPUT_WAIT =>
          case conv_integer(cStateCounter) is
            when 0 =>
              cInputReq <= '1';
              cStateCounter <= cStateCounter + 1;
            when 1 =>
              if cInputBusy = '1' then
                cStateCounter <= cStateCounter + 1;
              else
                cInputReq <= '0';
              end if;  
            when 2 =>
              cInputReq <= '0';
              if cInputBusy = '0' then
                cState <= UPDATE;
                cInputX <= cUserInputX;
                cInputY <= cUserInputY;
                cStateCounter <= (others => '0');
              end if;
            when others => null;
          end case;
        when UPDATE =>
          -----------------------------------------------------------------
          -- ɐ΂̂_w肵pX
          -----------------------------------------------------------------
          if cBlackData(conv_integer(cInputX&cInputY)) = '1' or cWhiteData(conv_integer(cInputX&cInputY)) = '1' then
            cStateCounter <= (others => '0');
            cState <= PRINT;
            cGameTurn <= cOppositeGameTurn;
          -----------------------------------------------------------------
          -- ΂Ȃꏊw肵蒼
          -----------------------------------------------------------------
          elsif cReversiResultFlag = '0' then
            cStateCounter <= (others => '0');
            cState <= INPUT_WAIT;
          -----------------------------------------------------------------
          -- LȒu΂ɑ΂鏈
          -----------------------------------------------------------------
          else
            -- ΂u
            if cGameTurn = BLACK then
              cBlackData(conv_integer(cInputX&cInputY)) <= '1';
            elsif cGameTurn = WHITE then
              cWhiteData(conv_integer(cInputX&cInputY)) <= '1';
            end if;
            -- Ԃ
            for i in 0 to 63 loop
              if cReversiResultData(i) = '1' then
                if cGameTurn = BLACK then
                  cBlackData(i) <= '1';
                  cWhiteData(i) <= '0';
                elsif cGameTurn = WHITE then
                  cBlackData(i) <= '0';
                  cWhiteData(i) <= '1';
                end if;
              end if;
            end loop;
            cGameTurn <= cOppositeGameTurn;
            cStateCounter <= (others => '0');
            cState <= PRINT;
          end if;
        when COMP =>
          case conv_integer(cStateCounter) is
            when 0 =>
              cInputY <= cXORSHIFT(2 downto 0);
              cStateCounter <= cStateCounter + 1;
            when 1 =>
              cInputX <= cXORSHIFT(2 downto 0);
              cStateCounter <= cStateCounter + 1;
            when 2 =>
              cStateCounter <= (others => '0');
              if (cBlackData(conv_integer(cInputX&cInputY)) = '0' and cWhiteData(conv_integer(cInputX&cInputY)) = '0') and cReversiResultFlag = '1' then
                -- LȎIĂ
                cSelectCounter <= (others => '0');
                cState <= UPDATE;
              else
--                -- ȎIĂ
--                if conv_integer(cSelectCounter) >= MAX_TRY_NUM then
--                  -- ő厎s񐔂ɓB
--                  cSelectCounter <= (others => '0');
--                  cGameTurn <= cOppositeGameTurn;
--                  cState <= PRINT;
--                else
--                  cSelectCounter <= cSelectCounter + 1;
--                end if;
              end if;
            when others =>
              cState <= IDLE;
          end case;
        when others =>
          cState <= IDLE;
      end case;
    end if;    
  end process;

  process(cReversiResultData)
    variable flag : std_logic;
  begin
    flag := '0';
    for i in 0 to 63 loop
      if (cReversiResultData(i) = '1') then
        flag := '1';
      end if;
    end loop;  -- i
    cReversiResultFlag <= flag;
  end process;

end rtl;
